@@ -12,7 +12,7 @@ use AnyEvent::Finger::Request;
 use AnyEvent::Finger::Response;
 
 # ABSTRACT: Simple asynchronous finger server
-our $VERSION = '0.01'; # VERSION
+our $VERSION = '0.02'; # VERSION
 
 
 sub new
@@ -21,10 +21,11 @@ sub new
   my $args     = ref $_[0] eq 'HASH' ? (\%{$_[0]}) : ({@_});
   bless {
     hostname     => $args->{hostname},  
-    port         => $args->{port}     // 79,
-    on_error     => $args->{on_error} // sub { carp $_[0] },
+    port         => $args->{port}         // 79,
+    on_error     => $args->{on_error}     // sub { carp $_[0] },
+    on_bind      => $args->{on_bind}      // sub { },
     forward_deny => $args->{forward_deny} // 0,
-    forward      => $args->{forward} // 0,
+    forward      => $args->{forward}      // 0,
   }, $class;
 }
 
@@ -38,7 +39,7 @@ sub start
   croak "already started" if $self->{guard};
 
   $args->{$_} //= $self->{$_}
-    for qw( hostname port on_error forward forward_deny );
+    for qw( hostname port on_error on_bind forward forward_deny );
 
   my $forward = $args->{forward} // $self->{forward};
   if($forward)
@@ -122,21 +123,14 @@ sub start
     });
   };
   
-  if($args->{port} == 0)
-  {
-    my $done = AnyEvent->condvar;
-    $self->{guard} = tcp_server $args->{hostname}, undef, $cb, sub {
-      my($fh, $host, $port) = @_;
-      $self->{bindport} = $port;
-      $done->send;
-    };
-    $done->recv;
-  }
-  else
-  {
-    $self->{guard} = tcp_server $args->{hostname}, $args->{port}, $cb;
-    $self->{bindport} = $self->{port};
-  }
+  my $port = $args->{port};
+  undef $port if $port == 0;
+  
+  $self->{guard} = tcp_server $args->{hostname}, $port, $cb, sub {
+    my($fh, $host, $port) = @_;
+    $self->{bindport} = $port;
+    $args->{on_bind}->();
+  };
   
   $self;
 }
@@ -165,7 +159,7 @@ AnyEvent::Finger::Server - Simple asynchronous finger server
 
 =head1 VERSION
 
-version 0.01
+version 0.02
 
 =head1 SYNOPSIS
 
@@ -233,6 +227,13 @@ Passes the error string as the first argument to the callback.
 
 =item *
 
+on_bind
+
+A callback subref to be called when the port number is known.  This is
+useful when ephemeral port is used but other parts of the code depend on it.
+
+=item *
+
 forward_deny (0)
 
 Deny forward requests, (for example: C<finger@host1@host2@...> style requests).  
@@ -294,7 +295,10 @@ is called.
 
 The bind port.  If port is set to zero in the constructor or on
 start, then an ephemeral port will be used, and you can get the
-port number here.
+port number here.  This value is not available until the socket
+has been allocated and bound to a port, so if you need this
+value after calling C<start> but before any clients have connected
+use the C<on_bind> callback.
 
 =head2 $server-E<gt>stop
 
